@@ -12,6 +12,11 @@ from .io.file import open_file
 from .io.sequences import read_sequences
 from .io.vcf import is_vcf, read_vcf
 
+
+DELIMITER = '\t'
+ID_COLUMN = 'strain'
+
+
 def register_parser(parent_subparsers):
     parser = parent_subparsers.add_parser("index", help=__doc__)
     parser.add_argument('--sequences', '-s', required=True, help="sequences in FASTA or VCF formats. Augur will summarize the content of FASTA sequences and only report the names of strains found in a given VCF.")
@@ -26,9 +31,9 @@ def index_vcf(vcf_path, index_path):
 
     Parameters
     ----------
-    vcf_path : str or Path-like
+    vcf_path : str or `os.PathLike`
         path to a VCF file to index.
-    index_path : str or Path-like
+    index_path : str or `os.PathLike`
         path to a tab-delimited file containing the composition details for each
         sequence in the given input file.
 
@@ -42,10 +47,10 @@ def index_vcf(vcf_path, index_path):
     num_of_seqs = 0
 
     with open_file(index_path, 'wt') as out_file:
-        tsv_writer = csv.writer(out_file, delimiter = '\t')
+        tsv_writer = csv.writer(out_file, delimiter = DELIMITER)
 
         #write header i output file
-        header = ['strain']
+        header = [ID_COLUMN]
         tsv_writer.writerow(header)
 
         for record in strains:
@@ -63,7 +68,7 @@ def index_sequence(sequence, values):
     sequence : Bio.SeqRecord.SeqRecord
         sequence record to index.
 
-    values : list of sets of str
+    values : list of set of str
         values to count; sets must be non-overlapping and contain only
         single-character, lowercase strings
 
@@ -74,7 +79,8 @@ def index_sequence(sequence, values):
         for the given values, and a final column with the number of characters
         that didn't match any of those in the given values.
 
-
+    Examples
+    --------
     >>> other_IUPAC = {'r', 'y', 's', 'w', 'k', 'm', 'd', 'h', 'b', 'v'}
     >>> values = [{'a'},{'c'},{'g'},{'t'},{'n'}, other_IUPAC, {'-'}, {'?'}]
     >>> sequence_a = Bio.SeqRecord.SeqRecord(seq=Bio.Seq.Seq("ACTGN-?XWN"), id="seq_A")
@@ -153,10 +159,10 @@ def index_sequences(sequences_path, sequence_index_path):
 
     Parameters
     ----------
-    sequences_path : str or Path-like
+    sequences_path : str or `os.PathLike`
         path to a sequence file to index.
 
-    sequence_index_path : str or Path-like
+    sequence_index_path : str or `os.PathLike`
         path to a tab-delimited file containing the composition details for each
         sequence in the given input file.
 
@@ -182,7 +188,7 @@ def index_sequences(sequences_path, sequence_index_path):
         tsv_writer = csv.writer(out_file, delimiter = '\t', lineterminator='\n')
 
         #write header i output file
-        header = ['strain', 'length']+labels+['invalid_nucleotides']
+        header = [ID_COLUMN, 'length']+labels+['invalid_nucleotides']
         tsv_writer.writerow(header)
 
         for record in seqs:
