@@ -1,5 +1,6 @@
 import csv
 import os
+from typing import Iterable
 import pandas as pd
 import pyfastx
 import sys
@@ -12,28 +13,45 @@ from augur.types import DataErrorMethod
 from .file import open_file
 
 
-def read_metadata(metadata_file, id_columns=("strain", "name"), chunk_size=None):
-    """Read metadata from a given filename and into a pandas `DataFrame` or
+DEFAULT_DELIMITERS = (',', '\t')
+
+DEFAULT_ID_COLUMNS = ("strain", "name")
+
+METADATA_DATE_COLUMN = 'date'
+
+
+class InvalidDelimiter(Exception):
+    pass
+
+
+def read_metadata(metadata_file, delimiters=DEFAULT_DELIMITERS, id_columns=DEFAULT_ID_COLUMNS, chunk_size=None):
+    r"""Read metadata from a given filename and into a pandas `DataFrame` or
     `TextFileReader` object.
 
     Parameters
     ----------
     metadata_file : str
         Path to a metadata file to load.
-    id_columns : list[str]
+    delimiters : list of str
+        List of possible delimiters to check for between columns in the metadata.
+        Only one delimiter will be inferred.
+    id_columns : list of str
         List of possible id column names to check for, ordered by priority.
+        Only one id column will be inferred.
     chunk_size : int
         Size of chunks to stream from disk with an iterator instead of loading the entire input file into memory.
 
     Returns
     -------
-    pandas.DataFrame or pandas.TextFileReader
+    pandas.DataFrame or `pandas.io.parsers.TextFileReader`
 
     Raises
     ------
-    KeyError :
+    KeyError
         When the metadata file does not have any valid index columns.
 
+    Examples
+    --------
 
     For standard use, request a metadata file and get a pandas DataFrame.
 
@@ -58,10 +76,11 @@ def read_metadata(metadata_file, id_columns=("strain", "name"), chunk_size=None)
 
     """
     kwargs = {
-        "sep": None,
-        "engine": "python",
+        "sep": _get_delimiter(metadata_file, delimiters),
+        "engine": "c",
         "skipinitialspace": True,
         "na_filter": False,
+        "low_memory": False,
     }
 
     if chunk_size:
@@ -93,7 +112,10 @@ def read_metadata(metadata_file, id_columns=("strain", "name"), chunk_size=None)
     # of having its type inferred. This latter argument allows users to provide
     # numerical ids that don't get converted to numbers by pandas.
     kwargs["index_col"] = index_col
-    kwargs["dtype"] = {index_col: "string"}
+    kwargs["dtype"] = {
+        index_col: "string",
+        METADATA_DATE_COLUMN: "string"
+    }
 
     return pd.read_csv(
         metadata_file,
@@ -101,7 +123,7 @@ def read_metadata(metadata_file, id_columns=("strain", "name"), chunk_size=None)
     )
 
 
-def read_table_to_dict(table, duplicate_reporting=DataErrorMethod.ERROR_FIRST, id_column=None):
+def read_table_to_dict(table, delimiters, duplicate_reporting=DataErrorMethod.ERROR_FIRST, id_column=None):
     """
     Read rows from *table* file and yield each row as a single dict.
 
@@ -113,6 +135,10 @@ def read_table_to_dict(table, duplicate_reporting=DataErrorMethod.ERROR_FIRST, i
     table: str
         Path to a CSV or TSV file or IO buffer
 
+    delimiters : list of str
+        List of possible delimiters to check for between columns in the metadata.
+        Only one delimiter will be inferred.
+
     duplicate_reporting: DataErrorMethod, optional
         How should duplicate records be reported
 
@@ -127,13 +153,12 @@ def read_table_to_dict(table, duplicate_reporting=DataErrorMethod.ERROR_FIRST, i
 
     Raises
     ------
-    AugurError:
+    AugurError
         Raised for any of the following reasons:
         1. There are parsing errors from the csv standard library
         2. The provided *id_column* does not exist in the *metadata*
         3. The *duplicate_reporting* method is set to ERROR_FIRST or ERROR_ALL and duplicate(s) are found
     """
-    valid_delimiters = [',', '\t']
     seen_ids = set()
     duplicate_ids = set()
     with open_file(table) as handle:
@@ -147,12 +172,13 @@ def read_table_to_dict(table, duplicate_reporting=DataErrorMethod.ERROR_FIRST, i
             handle = chain(table_sample_file, handle)
 
         try:
-            dialect = csv.Sniffer().sniff(table_sample, valid_delimiters)
-        except csv.Error as err:
-            raise AugurError(
-                f"Could not determine the delimiter of {table!r}. "
-                "File must be a CSV or TSV."
-            ) from err
+            # Note: this sort of duplicates _get_delimiter(), but it's easier if
+            # this is separate since it handles non-seekable buffers.
+            dialect = csv.Sniffer().sniff(table_sample, delimiters)
+        except csv.Error as error:
+            # This assumes all csv.Errors imply a delimiter issue. That might
+            # change in a future Python version.
+            raise InvalidDelimiter from error
 
         metadata_reader = csv.DictReader(handle, dialect=dialect)
         if duplicate_reporting is DataErrorMethod.SILENT:
@@ -194,7 +220,7 @@ def read_table_to_dict(table, duplicate_reporting=DataErrorMethod.ERROR_FIRST, i
             raise ValueError(f"Encountered unhandled duplicate reporting method: {duplicate_reporting!r}")
 
 
-def read_metadata_with_sequences(metadata, fasta, seq_id_column, seq_field='sequence',
+def read_metadata_with_sequences(metadata, metadata_delimiters, fasta, seq_id_column, seq_field='sequence',
     unmatched_reporting=DataErrorMethod.ERROR_FIRST, duplicate_reporting=DataErrorMethod.ERROR_FIRST):
     """
     Read rows from *metadata* file and yield each row as a single dict that has
@@ -224,6 +250,9 @@ def read_metadata_with_sequences(metadata, fasta, seq_id_column, seq_field='sequ
     metadata: str
         Path to a CSV or TSV metadata file
 
+    metadata_delimiters : list of str
+        List of possible delimiters to check for between columns in the metadata.
+
     fasta: str
         Path to a plain or gzipped FASTA file
 
@@ -286,7 +315,7 @@ def read_metadata_with_sequences(metadata, fasta, seq_id_column, seq_field='sequ
 
     # Silencing duplicate reporting here because we will need to handle duplicates
     # in both the metadata and FASTA files after processing all the records here.
-    for record in read_table_to_dict(metadata, duplicate_reporting=DataErrorMethod.SILENT):
+    for record in read_table_to_dict(metadata, metadata_delimiters, duplicate_reporting=DataErrorMethod.SILENT):
         seq_id = record.get(seq_id_column)
 
         if seq_id is None:
@@ -391,7 +420,7 @@ def write_records_to_tsv(records, output_file):
 
     Parameters
     ----------
-    records: iterator[dict]
+    records: iterable of dict
         Iterator that yields dict that contains sequences
 
     output_file: str
@@ -424,3 +453,20 @@ def write_records_to_tsv(records, output_file):
 
         for record in records:
             tsv_writer.writerow(record)
+
+
+def _get_delimiter(path: str, valid_delimiters: Iterable[str]):
+    """Get the delimiter of a file given a list of valid delimiters."""
+
+    for delimiter in valid_delimiters:
+        if len(delimiter) != 1:
+            raise AugurError(f"Delimiters must be single-character strings. {delimiter!r} does not satisfy that condition.")
+
+    with open_file(path) as file:
+        try:
+            # Infer the delimiter from the first line.
+            return csv.Sniffer().sniff(file.readline(), "".join(valid_delimiters)).delimiter
+        except csv.Error as error:
+            # This assumes all csv.Errors imply a delimiter issue. That might
+            # change in a future Python version.
+            raise InvalidDelimiter from error
