@@ -32,10 +32,11 @@ class TiterCollection(object):
 
         Returns
         -------
-        tuple (dict, list, list)
+        tuple
             tuple of a dict of titer measurements, list of strains, list of sources
 
-
+        Examples
+        --------
         >>> measurements, strains, sources = TiterCollection.load_from_file("tests/data/titer_model/h3n2_titers_subset.tsv")
         >>> type(measurements)
         <class 'dict'>
@@ -139,7 +140,7 @@ class TiterCollection(object):
 
         Parameters
         ----------
-        titers : defaultdict
+        titers : collections.defaultdict
             titer measurements indexed by test, reference,
             and serum
 
@@ -148,7 +149,8 @@ class TiterCollection(object):
         dict
             number of measurements per strain
 
-
+        Examples
+        --------
         >>> measurements, strains, sources = TiterCollection.load_from_file("tests/data/titer_model/h3n2_titers_subset.tsv")
         >>> titer_counts = TiterCollection.count_strains(measurements)
         >>> titer_counts["A/Acores/11/2013"]
@@ -184,7 +186,8 @@ class TiterCollection(object):
             reduced dictionary of titer measurements containing only those were
             test and reference virus are part of the strain list
 
-
+        Examples
+        --------
         >>> measurements, strains, sources = TiterCollection.load_from_file("tests/data/titer_model/h3n2_titers_subset.tsv")
         >>> len(measurements)
         11
@@ -214,10 +217,8 @@ class TiterCollection(object):
 
         Parameters
         ----------
-        titers : TYPE
-            Description
+        titers
         **kwargs
-            Description
         """
         self.kwargs = kwargs
 
@@ -248,15 +249,8 @@ class TiterCollection(object):
 
         Parameters
         ----------
-        ref : TYPE
-            Description
-        val : TYPE
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
+        ref
+        val
         '''
         consensus_func = np.mean
         return consensus_func(np.log2(self.autologous_titers[ref]['val'])) \
@@ -321,6 +315,8 @@ class TiterCollection(object):
         make lists of reference viruses, test viruses and sera
         (there are often multiple sera per reference virus)
 
+        Examples
+        --------
         >>> measurements, strains, sources = TiterCollection.load_from_file("tests/data/titer_model/h3n2_titers_subset.tsv")
         >>> titers = TiterCollection(measurements)
         >>> sera, ref_strains, test_strains = titers.strain_census(measurements)
@@ -333,13 +329,7 @@ class TiterCollection(object):
 
         Parameters
         ----------
-        titers : TYPE
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
+        titers
         """
         sera = set()
         ref_strains = set()
@@ -459,15 +449,10 @@ class TiterModel(object):
         Parameters
         ----------
         method : str, optional
-            Description
         lam_drop : float, optional
-            Description
         lam_pot : float, optional
-            Description
         lam_avi : float, optional
-            Description
         **kwargs
-            Description
         '''
         self.lam_pot = lam_pot
         self.lam_avi = lam_avi
@@ -510,18 +495,9 @@ class TiterModel(object):
         Parameters
         ----------
         plot : bool, optional
-            Description
         cutoff : float, optional
-            Description
         validation_set : None, optional
-            Description
         fname : None, optional
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         from scipy.stats import linregress, pearsonr
         if validation_set is None:
@@ -592,11 +568,6 @@ class TiterModel(object):
         during visualization, we need the average distance of a test virus from
         a reference virus across sera. hence the hierarchy [ref][test][serum]
         NOTE: this uses node.name instead of node.clade
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         def dstruct():
             return defaultdict(dict)
@@ -614,11 +585,6 @@ class TiterModel(object):
         compile a json structure containing potencies for visualization
         we need rapid access to all sera for a given reference virus, hence
         the structure is organized by [ref][serum]
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         potency_json = defaultdict(dict)
         for (ref_clade, serum), val in self.serum_potency.items():
@@ -639,11 +605,6 @@ class TiterModel(object):
     def compile_virus_effects(self):
         '''
         compile a json structure containing virus_effects for visualization
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         return {test_vir:np.round(val,TITER_ROUND) for test_vir, val in self.virus_effect.items()}
 
@@ -654,11 +615,6 @@ class TiterModel(object):
     def fit_l1reg(self):
         '''
         regularize genetic parameters with an l1 norm regardless of sign
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         try:
             from cvxopt import matrix, solvers
@@ -718,11 +674,6 @@ class TiterModel(object):
 
     def fit_nnl1reg(self):
         '''l1 regularization of titer drops with non-negativity constraints
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         try:
             from cvxopt import matrix, solvers
@@ -807,15 +758,8 @@ class TreeModel(TiterModel):
 
         Parameters
         ----------
-        n : TYPE
-            Description
+        n
         **kwargs
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
 
         model_performance = []
@@ -843,15 +787,8 @@ class TreeModel(TiterModel):
 
         Parameters
         ----------
-        v1 : TYPE
-            Description
-        v2 : TYPE
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
+        v1
+        v2
         '''
         if v1 in self.strain_lookup and v2 in self.strain_lookup:
             p1 = [self.strain_lookup[v1]]
@@ -881,7 +818,6 @@ class TreeModel(TiterModel):
         Parameters
         ----------
         criterium : None, optional
-            Description
         '''
         if criterium is None:
             criterium = lambda x:True
@@ -1029,15 +965,8 @@ class SubstitutionModel(TiterModel):
 
         Parameters
         ----------
-        strain1 : TYPE
-            Description
-        strain2 : TYPE
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
+        strain1
+        strain2
         '''
         if strain1 in self.sequences and strain2 in self.sequences:
             muts = []
@@ -1084,7 +1013,6 @@ class SubstitutionModel(TiterModel):
         Parameters
         ----------
         colin_thres : int, optional
-            Description
         '''
         seq_graph = []
         titer_dist = []
@@ -1134,8 +1062,7 @@ class SubstitutionModel(TiterModel):
 
         Parameters
         ----------
-        colin_thres : TYPE
-            Description
+        colin_thres
         '''
         TT = self.design_matrix[:,:self.genetic_params].T
         mutation_clusters = []
@@ -1170,7 +1097,6 @@ class SubstitutionModel(TiterModel):
         Parameters
         ----------
         **kwargs
-            Description
         '''
         self._train(**kwargs)
         for mi, mut in enumerate(self.relevant_muts):
@@ -1195,12 +1121,6 @@ class SubstitutionModel(TiterModel):
         Parameters
         ----------
         cutoff : float, optional
-            Description
-
-        Returns
-        -------
-        TYPE
-            Description
         '''
         return {mut[0]+':'+mut[1]:np.round(val,int(-np.log10(cutoff)))
                 for mut, val in self.substitution_effect.items() if val>cutoff}
@@ -1211,11 +1131,11 @@ class SubstitutionModel(TiterModel):
 
         Parameters
         ----------
-        tree : Bio.Phylo
+        tree : Bio.Phylo.BaseTree.Tree
 
         Returns
         -------
-        Bio.Phylo
+        Bio.Phylo.BaseTree.Tree
             input tree instance with nodes annotated by per-branch and
             cumulative antigenic advance attributes `dTiterSub` and
             `cTiterSub`
@@ -1236,16 +1156,3 @@ class SubstitutionModel(TiterModel):
                 child.cTiterSub = node.cTiterSub + child.dTiterSub
 
         return tree
-
-
-if __name__=="__main__":
-    # test tree model (assumes there is a tree called flu in memory...)
-    ttm = TreeModel(flu.tree.tree, flu.titers)
-    ttm.prepare(training_fraction=0.8)
-    ttm.train(method='nnl1reg')
-    ttm.validate(plot=True)
-
-    tsm = SubstitutionModel(flu.tree.tree, flu.titers)
-    tsm.prepare(training_fraction=0.8)
-    tsm.train(method='nnl1reg')
-    tsm.validate(plot=True)
