@@ -6,7 +6,8 @@ import numpy as np
 from collections import defaultdict
 import os, sys
 import pandas as pd
-from .io.metadata import read_metadata
+from .errors import AugurError
+from .io.metadata import DEFAULT_DELIMITERS, DEFAULT_ID_COLUMNS, InvalidDelimiter, read_metadata
 from .utils import write_json, get_json_name
 TINY = 1e-12
 
@@ -34,7 +35,7 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
 
     Returns
     -------
-    T : Phylo.Tree
+    T : Bio.Phylo.BaseTree.Tree
         Biophyton tree
     gtr : treetime.GTR
         GTR model
@@ -98,16 +99,13 @@ def mugration_inference(tree=None, seq_meta=None, field='country', confidence=Tr
 
 
 def register_parser(parent_subparsers):
-    """Add subcommand specific arguments
-
-    Parameters
-    ----------
-    parser : argparse
-        subcommand argument parser
-    """
     parser = parent_subparsers.add_parser("traits", help=__doc__)
     parser.add_argument('--tree', '-t', required=True, help="tree to perform trait reconstruction on")
-    parser.add_argument('--metadata', required=True, metavar="FILE", help="table with metadata, as CSV or TSV")
+    parser.add_argument('--metadata', required=True, metavar="FILE", help="table with metadata")
+    parser.add_argument('--metadata-delimiters', default=DEFAULT_DELIMITERS, nargs="+",
+                        help="delimiters to accept when reading a metadata file. Only one delimiter will be inferred.")
+    parser.add_argument('--metadata-id-columns', default=DEFAULT_ID_COLUMNS, nargs="+",
+                        help="names of possible metadata columns containing identifier information, ordered by priority. Only one ID column will be inferred.")
     parser.add_argument('--weights', required=False, help="tsv/csv table with equilibrium probabilities of discrete states")
     parser.add_argument('--columns', required=True, nargs='+',
                         help='metadata fields to perform discrete reconstruction on')
@@ -129,11 +127,21 @@ def run(args):
 
     Parameters
     ----------
-    args : namespace
+    args : argparse.Namespace
         command line arguments are parsed by argparse
     """
     tree_fname = args.tree
-    traits = read_metadata(args.metadata)
+    try:
+        traits = read_metadata(
+            args.metadata,
+            delimiters=args.metadata_delimiters,
+            id_columns=args.metadata_id_columns)
+    except InvalidDelimiter:
+        raise AugurError(
+                f"Could not determine the delimiter of {args.metadata!r}. "
+                f"Valid delimiters are: {args.metadata_delimiters!r}. "
+                "This can be changed with --metadata-delimiters."
+            )
 
     from Bio import Phylo
     T = Phylo.read(tree_fname, 'newick')
