@@ -1,5 +1,16 @@
 """
 Translate gene regions from nucleotides to amino acids.
+
+Translates nucleotide sequences of nodes in a tree to amino acids for gene
+regions of the annotated features of the provided reference sequence.
+Each node then gets assigned a list of amino acid mutations for any position
+that has a mismatch between its own amino acid sequence and its parent's sequence.
+The reference amino acid sequences, genome annotations, and node amino acid
+mutations are output to a node-data JSON file.
+
+.. note::
+
+    The mutation positions in the node-data JSON are one-based.
 """
 
 import os, sys
@@ -22,6 +33,8 @@ def safe_translate(sequence, report_exceptions=False):
     Optionally, returns a tuple of the translated sequence and whether an
     exception was raised during initial translation.
 
+    Examples
+    --------
     >>> safe_translate("ATG")
     'M'
     >>> safe_translate("ATGGT-")
@@ -351,10 +364,10 @@ def run(args):
 
     ## load features; only requested features if genes given
     features = load_features(args.reference_sequence, genes)
-    print("Read in {} features from reference sequence file".format(len(features)))
     if features is None:
         print("ERROR: could not read features of reference sequence file")
         return 1
+    print("Read in {} features from reference sequence file".format(len(features)))
 
     ### translate every feature - but not 'nuc'!
     translations = {}
@@ -384,7 +397,7 @@ def run(args):
                               'type':feat.type,
                               'start':int(feat.location.start)+1,
                               'end':int(feat.location.end),
-                              'strand': '+' if feat.location.strand else '-'}
+                              'strand': {+1:'+', -1:'-', 0:'?', None:None}[feat.location.strand]}
     if is_vcf: #need to add our own nuc
         annotations['nuc'] = {'seqid':args.reference_sequence,
                               'type':feat.type,
