@@ -15,6 +15,8 @@ from .__version__ import __version__
 
 from augur.io.file import open_file
 
+from augur.types import ValidationMode
+
 from augur.util_support.color_parser import ColorParser
 from augur.util_support.node_data_reader import NodeDataReader
 
@@ -56,7 +58,7 @@ def read_tree(fname, min_terminals=3):
 
     Returns
     -------
-    Bio.Phylo :
+    Bio.Phylo.BaseTree.Tree :
         BioPython tree instance
 
     """
@@ -87,8 +89,8 @@ def read_tree(fname, min_terminals=3):
     return T
 
 
-def read_node_data(fnames, tree=None, skip_validation=False):
-    return NodeDataReader(fnames, tree, skip_validation).read()
+def read_node_data(fnames, tree=None, validation_mode=ValidationMode.ERROR):
+    return NodeDataReader(fnames, tree, validation_mode).read()
 
 
 def write_json(data, file_name, indent=(None if os.environ.get("AUGUR_MINIFY_JSON") else 2), include_version=True):
@@ -125,11 +127,14 @@ def write_json(data, file_name, indent=(None if os.environ.get("AUGUR_MINIFY_JSO
         data["generated_by"] = {"program": "augur", "version": get_augur_version()}
     with open(file_name, 'w', encoding='utf-8') as handle:
         sort_keys = False if isinstance(data, OrderedDict) else True
-        json.dump(data, handle, indent=indent, sort_keys=sort_keys, cls=NumpyJSONEncoder)
+        json.dump(data, handle, indent=indent, sort_keys=sort_keys, cls=AugurJSONEncoder)
 
 
-class NumpyJSONEncoder(json.JSONEncoder):
-    """A custom JSONEncoder subclass to serialize additional numpy data types."""
+class AugurJSONEncoder(json.JSONEncoder):
+    """
+    A custom JSONEncoder subclass to serialize data types used for various data
+    stored in dictionary format.
+    """
     def default(self, obj):
         if isinstance(obj, np.integer):
             return int(obj)
@@ -137,6 +142,8 @@ class NumpyJSONEncoder(json.JSONEncoder):
             return float(obj)
         if isinstance(obj, np.ndarray):
             return obj.tolist()
+        if isinstance(obj, pd.Series):
+            return obj.tolist()
         return super().default(obj)
 
 
@@ -150,11 +157,7 @@ def load_features(reference, feature_names=None):
     features = {}
     if '.gff' in reference.lower():
         #looks for 'gene' and 'gene' as best for TB
-        try:
-            from BCBio import GFF #Package name is confusing - tell user exactly what they need!
-        except ImportError:
-            print("ERROR: Package BCBio.GFF not found! Please install using \'pip install bcbio-gff\' before re-running.")
-            return None
+        from BCBio import GFF
         limit_info = dict( gff_type = ['gene', 'source'] )
 
         with open(reference, encoding='utf-8') as in_handle:
@@ -286,6 +289,7 @@ def available_cpu_cores(fallback: int = 1) -> int:
         # 24 total cores may only allow our process to use 12.  If we tried to
         # naively use all 24, we'd end up with two threads across the 12 cores.
         # This would degrade performance rather than improve it!
+        assert hasattr(os, "sched_getaffinity")
         return len(os.sched_getaffinity(0))
     except:
         # cpu_count() returns None if the value is indeterminable.
@@ -320,6 +324,8 @@ def get_parent_name_by_child_name_for_tree(tree):
 def annotate_parents_for_tree(tree):
     """Annotate each node in the given tree with its parent.
 
+    Examples
+    --------
     >>> import io
     >>> tree = Bio.Phylo.read(io.StringIO("(A, (B, C))"), "newick")
     >>> not any([hasattr(node, "parent") for node in tree.find_clades()])
@@ -345,6 +351,9 @@ def json_to_tree(json_dict, root=True, parent_cumulative_branch_length=None):
 
     Assigns links back to parent nodes for the root of the tree.
 
+    Examples
+    --------
+
     Test opening a JSON from augur export v1.
 
     >>> import json
@@ -458,7 +467,7 @@ def read_bed_file(bed_file):
 
     Returns
     -------
-    list[int]:
+    list of int:
         Sorted list of unique zero-indexed sites
     """
     mask_sites = []
@@ -489,7 +498,7 @@ def read_mask_file(mask_file):
 
     Returns
     -------
-    list[int]:
+    list of int:
         Sorted list of unique zero-indexed sites
     """
     mask_sites = []
@@ -515,7 +524,7 @@ def load_mask_sites(mask_file):
 
     Returns
     -------
-    list[int]
+    list of int
         Sorted list of unique zero-indexed sites
     """
     if mask_file.lower().endswith(".bed"):
@@ -545,7 +554,7 @@ def read_strains(*files, comment_char="#"):
 
     Parameters
     ----------
-    files : one or more str
+    files : iterable of str
         one or more names of text files with one strain name per line
 
     Returns
