@@ -2,6 +2,7 @@ import json
 import numpy as np
 from pathlib import Path
 from unittest.mock import patch
+import pandas as pd
 
 import pytest
 
@@ -93,12 +94,13 @@ class TestUtils:
         assert len(strains) == 3
         assert "strain1" in strains
 
-    def test_write_json_numpy_types(self, tmpdir):
-        """write_json should be able to serialize numpy data types."""
+    def test_write_json_data_types(self, tmpdir):
+        """write_json should be able to serialize various data types."""
         data = {
             'int': np.int64(1),
             'float': np.float64(2.0),
-            'array': np.array([3,4,5])
+            'array': np.array([3,4,5]),
+            'series': pd.Series([6,7,8])
         }
         file = Path(tmpdir) / Path("data.json")
         utils.write_json(data, file, include_version=False)
@@ -106,5 +108,6 @@ class TestUtils:
             assert json.load(f) == {
                 'int': 1,
                 'float': 2.0,
-                'array': [3,4,5]
+                'array': [3,4,5],
+                'series': [6,7,8]
             }
